from lamin_logger import logger
from lnhub_rest.core.instance._init_instance import init_instance as init_instance_hub

from ._settings import settings


def register():
    """Register an instance on the hub."""
    isettings = settings.instance
    result = init_instance_hub(
        owner=isettings.owner,
        name=isettings.name,
        storage=isettings.storage.root_as_str,
        db=isettings.db,
        schema=isettings._schema_str,
    )
    if result == "instance-exists-already":
        logger.info("Instance already registered")
    elif isinstance(result, str):
        raise RuntimeError(f"Creating instance on hub failed:\n{result}")
    logger.success("Instance registered")



from lnhub_rest import __version__ as lnhub_rest_v
from packaging import version

if version.parse(lnhub_rest_v) != version.parse("0.8.2"):
    raise SystemExit("Please upgrade lnhub_rest: pip install lnhub_rest==0.8.2")



import sqlmodel as sqm

import lndb


def test_init_bionty():
    lndb.login(
        "testuser1@lamin.ai", password="cEvcwMJFX4OwbsYVaMt2Os6GxxGgDUlBGILs2RyS"
    )
    lndb.init(storage="test-bionty", schema="bionty")

    import lnschema_bionty

    with sqm.Session(lndb.settings.instance.engine) as session:
        session.exec(
            sqm.select(lnschema_bionty.dev.CurrentBiontyVersions).where(
                lnschema_bionty.dev.CurrentBiontyVersions.entity == "Gene"
            )
        ).one()



import os

import nox
from laminci import move_built_docs_to_docs_slash_project_slug, upload_docs_artifact
from laminci.nox import (
    build_docs,
    login_testuser1,
    login_testuser2,
    run_pre_commit,
    run_pytest,
)

nox.options.reuse_existing_virtualenvs = True


LAMIN_ENV = "prod"
if "GITHUB_REF_NAME" in os.environ:
    if os.environ["GITHUB_REF_NAME"] == "main":
        LAMIN_ENV = "prod"
    elif os.environ["GITHUB_REF_NAME"] == "staging":
        LAMIN_ENV = "staging"
elif "LAMIN_ENV" in os.environ:
    LAMIN_ENV = os.environ["LAMIN_ENV"]

env = {"LAMIN_ENV": LAMIN_ENV}


@nox.session
def lint(session: nox.Session) -> None:
    run_pre_commit(session)


@nox.session(python=["3.7", "3.8", "3.9", "3.10", "3.11"])
def build(session):
    session.install(".[dev,test]")
    login_testuser1(session, env=env)
    login_testuser2(session, env=env)
    run_pytest(session, env=env)
    build_docs(session)
    upload_docs_artifact()
    move_built_docs_to_docs_slash_project_slug()



from pathlib import Path
from typing import Union

from lamin_logger import logger
from lnhub_rest.core.storage._add_storage import add_storage as add_storage_hub

from lndb.dev.upath import UPath

from ._init_instance import register
from ._settings import settings
from .dev import deprecated
from .dev._settings_instance import InstanceSettings


class set:
    """Set properties of current instance."""

    @staticmethod
    def storage(root: Union[str, Path, UPath], **fs_kwargs):
        """Set storage.

        Args:
            root: `Union[str, Path, UPath]` - The new storage root, e.g., an S3 bucket.
            **fs_kwargs: Additional fsspec arguments for cloud root, e.g., profile.

        Example:

        >>> ln.setup.set.storage(
        >>>    "s3://some-bucket",
        >>>     profile="some_profile", # fsspec arg
        >>>     cache_regions=True # fsspec arg for s3
        >>> )
        """
        if settings.instance.dialect == "sqlite":
            logger.error("Can't set storage for sqlite instances.")
            return "set-storage-failed"

        new_isettings = InstanceSettings(
            owner=settings.instance.owner,
            name=settings.instance.name,
            storage_root=root,
            db=settings.instance.db,
            schema=settings.instance._schema_str,
        )

        new_isettings._persist()  # this also updates the settings object
        register(new_isettings, settings.user)
        if settings.instance.is_remote:
            add_storage_hub(root, account_handle=settings.instance.owner)

        settings.storage._set_fs_kwargs(**fs_kwargs)

        logger.success(f"Set storage {root}")


@deprecated("lndb.set.storage()")
def set_storage(storage: Union[str, Path, UPath]):
    """Deprecated in favor of `set.storage`."""
    set.storage(storage)
